/* abstract_crypto.h
 *
 * Interface to block cryptos and hash functions
 *
 * $Id: abstract_crypto.h,v 1.23 1999/11/28 17:53:49 nisse Exp $ */

/* lsh, an implementation of the ssh protocol
 *
 * Copyright (C) 1998 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LSH_ABSTRACT_CRYPTO_H_INCLUDED
#define LSH_ABSTRACT_CRYPTO_H_INCLUDED

#include "list.h"

/* Use the same instance struct for both hash functions and macs. This
 * is a little ugly. */
#define mac_instance_class hash_instance_class
#define mac_instance hash_instance
#define mac_size hash_size

#define GABA_DECLARE
#include "abstract_crypto.h.x"
#undef GABA_DECLARE

/* GABA:
   (class
     (name crypto_instance)
     (vars
       (block_size simple UINT32)
       ; Length must be a multiple of the block size.
       ; NOTE: src == dst is allowed, but no other overlaps.
       (crypt method void
              "UINT32 length" "const UINT8 *src" "UINT8 *dst")))
*/

#define CRYPT(instance, length, src, dst) \
((instance)->crypt((instance), (length), (src), (dst)))

#define CRYPTO_ENCRYPT 0
#define CRYPTO_DECRYPT 1

/* GABA:
   (class
     (name crypto_algorithm)
     (vars
       (block_size simple UINT32)
       (key_size simple UINT32)
       (iv_size simple UINT32)
       (make_crypt method (object crypto_instance)
                   "int mode" "const UINT8 *key" "const UINT8 *iv")))
*/

#define MAKE_CRYPT(crypto, mode, key, iv) \
((crypto)->make_crypt((crypto), (mode), (key), (iv)))     

#define MAKE_ENCRYPT(crypto, key, iv) \
     MAKE_CRYPT((crypto), CRYPTO_ENCRYPT, (key), (iv))

#define MAKE_DECRYPT(crypto, key, iv) \
     MAKE_CRYPT((crypto), CRYPTO_DECRYPT, (key), (iv))
     
/* FIXME: Hashes could use non-virtual methods. */
     
/* GABA:
   (class
     (name hash_instance)
     (vars
       (hash_size simple UINT32)
       (update method void 
	       "UINT32 length" "UINT8 *data")
       (digest method void "UINT8 *result")
       (copy method (object hash_instance))))
*/

#define HASH_UPDATE(instance, length, data) \
((instance)->update((instance), (length), (data)))

#define HASH_DIGEST(instance, result) \
((instance)->digest((instance), (result)))

#define HASH_COPY(instance) ((instance)->copy((instance)))

/* GABA:
   (class
     (name hash_algorithm)
     (vars
       (block_size simple UINT32)
       (hash_size simple UINT32)
       (make_hash method (object hash_instance))))
*/

#define MAKE_HASH(h) ((h)->make_hash((h)))

/* GABA:
   (class
     (name mac_algorithm)
     (vars
       (hash_size simple UINT32)
       (key_size simple UINT32)
       (make_mac method (object mac_instance) "const UINT8 *key")))
*/

#define MAKE_MAC(m, key) ((m)->make_mac((m), (key)))


/* GABA:
   (class
    (name signer)
    (vars
      ; Returns a non-spki signature
      (sign method (string)
            "UINT32 length" "UINT8 *data")
      ; Returns a signature sexp
      (sign_spki method (object sexp)
                 "struct sexp *hash" "struct sexp *principal"
                 "UINT32 length" "UINT8 *data")))
*/

#define SIGN(signer, length, data) ((signer)->sign((signer), (length), (data)))

#define SIGN_SPKI(signer, length, data) ((signer)->sign((signer), (length), (data)))

/* GABA:
   (class
     (name verifier)
     (vars
       (verify method int
       	       "UINT32 length" "UINT8 *data"
	       "UINT32 signature_length" "UINT8 * signature_data")
       ; Iterator points past the signature-tag
       (verify_spki method int
       	       "UINT32 length" "UINT8 *data"
	       "struct sexp_iterator *i")))
*/

#define VERIFY(verifier, length, data, slength, sdata) \
((verifier)->verify((verifier), (length), (data), (slength), (sdata)))

#define VERIFY_SPKI(verifier, length, data, i) \
((verifier)->verify((verifier), (length), (data), (i)))

/* GABA:
   (class
     (name signature_algorithm)
     (vars
       ; Iterators should point past the algorithm tag
       (make_signer method (object signer)
                    "struct sexp_iterator *i")
		    
       (make_verifier method (object verifier)
                      "struct sexp_iterator *i")))
*/

#define MAKE_SIGNER(a, i) \
((a)->make_signer((a), (i)))

#define MAKE_VERIFIER(a, i) \
((a)->make_verifier((a), (i)))


/* Combining block cryptos */

/* Example: To create a tripple DES cbc encryptor:
 *
 * struct crypto_algorithm des3_cbc
 *  = make_cbc(crypto_cascade(3, des_algorithm,
 *                               crypto_invert(des_algorithm)
 *                               des_algorithm, -1));
 */

struct crypto_algorithm *crypto_cbc(struct crypto_algorithm *inner);
struct crypto_algorithm *crypto_invert(struct crypto_algorithm *inner);
struct crypto_algorithm *crypto_cascadel(struct object_list *cascade);
struct crypto_algorithm *crypto_cascade(unsigned n, ...);

/* Utility functions */
UINT32 gcd(UINT32 x, UINT32 y);
UINT32 lcm(UINT32 x, UINT32 y);

/* Simple hashing */
struct lsh_string *
hash_string(struct hash_algorithm *a,
	    struct lsh_string *in,
	    int free);

#endif /* LSH_ABSTRACT_CRYPTO_H_INCLUDED */
